/*
 * Created on 2005/04/18
 */
package roomlayout;

import java.awt.Color;

/**
 * @author kwsk
 */
public class Piece
{
    int x, y, w, h;
    private int a; // 0 - 359
    private Color c;
    

    public Piece() {
        this.x = 400;
        this.y = 300;
        this.w = 100;
        this.h = 50;
        this.a = 0;
        this.c = Color.gray;
    }
    
    public Piece(int x, int y, int w, int h, int a, int c) {
        this.x = x;
        this.y = y;
        this.w = w;
        this.h = h;
        this.a = a;
        this.c = new Color(c);
    }
    
    public Piece(Piece p) {
        copyFrom(p);
    }
    
    public Piece(String s) {
        // toString̏o͂tH[}bgp[X
        String[] vals = s.split(",");
        x = Integer.parseInt(vals[0]); 
        y = Integer.parseInt(vals[1]);
        w = Integer.parseInt(vals[2]);
        h = Integer.parseInt(vals[3]);
        a = Integer.parseInt(vals[4]);
        setColor(vals[5]);
    }

    public String toString() {
        return x + "," + y + "," + w + "," + h + "," + a + "," + getColorString();
    }
    
    
    public int getAngle() {
        return a;
    }
    public void setAngle(int a) {
        this.a = normalizeDegrees(a);
    }
    public Color getColor() {
        return c;
    }
    public void setColor(Color c) {
        this.c = c;
    }
    
    public void setColor(String s) {
        c = new Color(Integer.parseInt(s, 16));
    }
    
    public String getColorString() {
        String colorString = Integer.toHexString(c.getRGB()).substring(2); // alphalƂ̂
        return colorString;
    }
    

    public void translate(int dx, int dy) {
        x += dx;
        y += dy;
    }
    
    public void rotate(int da) {
        a = normalizeDegrees(a + da);
    }
    
    public void rotateRadian(double theta) {
        a = normalizeDegrees((int)Math.toDegrees(Math.toRadians(a) + theta));
    }
    
    public int normalizeDegrees(int a) {
        a %= 360;
        if(a < 0) a+= 360;
        return a;
    }
    
    public boolean contains(int px, int py) {
        return contains(new Vector2D(px, py));
    }
    

    public Vector2D getVertex(int index) {
        double radian = Math.toRadians(a);
        int dx0 = (int)(   w * Math.cos(radian));
        int dy0 = (int)(   w * Math.sin(radian));
        int dx1 = (int)( - h * Math.sin(radian));
        int dy1 = (int)(   h * Math.cos(radian));
        
        int ox = x - dx0 / 2 - dx1 / 2;
        int oy = y - dy0 / 2 - dy1 / 2;

        if(index == 0) return new Vector2D(ox, oy);
        else if(index == 1) return new Vector2D(ox + dx0, oy + dy0);
        else if(index == 2) return new Vector2D(ox + dx0 + dx1, oy + dy0 + dy1);
        else if(index == 3) return new Vector2D(ox + dx1, oy + dy1);
        return null;
    }
    
    
    public boolean contains(Vector2D p) {
        Vector2D v0 = getVertex(0);
        Vector2D v1 = getVertex(1);
        Vector2D v2 = getVertex(2);
        Vector2D v3 = getVertex(3);
        
        return Vector2D.onTheRight(v0, v1, p)
        && Vector2D.onTheRight(v1, v2, p)
        && Vector2D.onTheRight(v2, v3, p)
        && Vector2D.onTheRight(v3, v0, p);
    }
    
    public void copyFrom(Piece p) {
        x = p.x;
        y = p.y;
        w = p.w;
        h = p.h;
        a = p.a;
        c = p.c;
    }
    
    public int error(Piece p) {
        // _S̈ʒû̂Q̘a
        int error = Integer.MAX_VALUE;
        for(int offset = 0 ; offset < 4 ; offset++) { // Ώ̐l
            int sum = 0;
            for(int i = 0 ; i < 4 ; i++) {
                int dx = getVertex(i).x - p.getVertex((i + offset) % 4).x;
                int dy = getVertex(i).y - p.getVertex((i + offset) % 4).y;
                sum += dx * dx + dy * dy;
            }
            error = Math.min(error, sum);
        }
        return error;
    }
}
