package roomlayout;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.Vector;

/**
 * @author kwsk
 */
public class RoomModel
{
    private Vector pieces;
    
    public RoomModel() {
        init();
        addPiece(300, 150, 100, 50, 90, 0xffff88);
        addPiece(200, 100, 100, 50, 0, 0xff88ff);
        addPiece(200, 400, 200, 50, 135, 0x88ffff);
        addPiece(500, 300, 200, 200, 0, 0x8888ff);
    }
    
    public RoomModel(String filename) {
        load(new File(filename));
    }
    
    public synchronized void init() {
        pieces = new Vector();
    }
    
    public synchronized void load(String s) {
        load(new File(s));
    }
    
    public synchronized void load(File file) {
        Log.println("Load: " + file.toString());
        init();
        try {
            BufferedReader reader = new BufferedReader(new FileReader(file));
            int id = 0;
            while(true) {
                String line = reader.readLine();
                if(line == null) break;
                addPiece(line);
            }
            reader.close();
        }
        catch(IOException e) {
            e.printStackTrace();
        }
        
    }

    public synchronized void save(String s) {
        save(new File(s));
    }

    public synchronized void save(File file) {
        Log.println("Save: " + file.toString());
        try {
            BufferedWriter writer = new BufferedWriter(new FileWriter(file));
            for(Iterator i = pieceIterator() ; i.hasNext() ; ) {
                String s = ((Piece)i.next()).toString() + "\n";
                writer.write(s);
            }
            writer.close();
        }
        catch(IOException e) {
            e.printStackTrace();
        }
    }

    public synchronized void addPiece(String s) {
        pieces.add(new Piece(s));
    }

    public synchronized void addPiece(int x, int y, int w, int h, int a, int c) {
        pieces.add(new Piece(x, y, w, h, a, c));
    }

    public synchronized void addPiece(Piece piece) {
        pieces.add(piece);
    }

    public synchronized void removePiece(Piece piece) {
        pieces.remove(piece);
    }
    
    public synchronized int getId(Piece piece) {
        return pieces.indexOf(piece);
    }

    public Iterator pieceIterator() {
        return pieces.iterator();
    }

    public boolean containsPiece(Piece p) {
        return pieces.contains(p);
    }
    
    public synchronized Piece pieceAt(int px, int py) {
        Piece target = null;
        for(Iterator i = pieceIterator() ; i.hasNext() ; ) {
            Piece piece = (Piece)i.next();
            if(piece.contains(px, py)) target = piece;
        }
        return target;
    }

    
    
    public synchronized int error(RoomModel model) {
        int error = 0;
        synchronized(model) {
            Iterator i = pieceIterator();
            Iterator j = model.pieceIterator();
            while(i.hasNext()) {
                Piece p0 = (Piece)i.next();
                Piece p1 = (Piece)j.next();
                error += p0.error(p1);
            }
        }
        return error;
    }

}
